import { Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { FormArray, FormControl, FormGroup } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { MatSelectChange } from '@angular/material/select';
import { ActivatedRoute, Router } from '@angular/router';
import { BehaviorSubject } from 'rxjs';
import { map } from 'rxjs/operators';

import { Printer } from '../core/printer';
import { Section } from '../core/section';
import { SectionPrinter } from '../core/section-printer';
import { ToasterService } from '../core/toaster.service';
import { ConfirmDialogComponent } from '../shared/confirm-dialog/confirm-dialog.component';

import { SectionPrinterDataSource } from './section-printer-datasource';
import { SectionPrinterService } from './section-printer.service';

@Component({
  selector: 'app-section-printer',
  templateUrl: './section-printer.component.html',
  styleUrls: ['./section-printer.component.css'],
})
export class SectionPrinterComponent implements OnInit {
  @ViewChild('section', { static: true }) sectionElement?: ElementRef;
  form: FormGroup<{
    section: FormControl<string>;
    saleCategories: FormArray<
      FormGroup<{
        saleCategory: FormControl<string>;
        printer: FormControl<string | null>;
        copies: FormControl<number>;
      }>
    >;
  }>;

  public listObservable = new BehaviorSubject<SectionPrinter[]>([]);
  dataSource: SectionPrinterDataSource = new SectionPrinterDataSource(this.listObservable);
  list: SectionPrinter[] = [];
  sections: Section[] = [];
  printers: Printer[] = [];
  sectionId = '';
  displayedColumns = ['saleCategory', 'printer', 'copies'];

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private toaster: ToasterService,
    private dialog: MatDialog,
    private ser: SectionPrinterService,
  ) {
    // Create form
    this.form = new FormGroup({
      section: new FormControl<string>('', { nonNullable: true }),
      saleCategories: new FormArray<
        FormGroup<{
          saleCategory: FormControl<string>;
          printer: FormControl<string | null>;
          copies: FormControl<number>;
        }>
      >([]),
    });
    route.params.pipe(map((p) => p['id'])).subscribe((x) => {
      this.sectionId = x;
    });
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { list: SectionPrinter[]; sections: Section[]; printers: Printer[] };
      this.sections = data.sections;
      this.printers = data.printers;
      this.showItem(data.list);
      this.dataSource = new SectionPrinterDataSource(this.listObservable);
      this.listObservable.next(this.list);
    });
  }

  showItem(list: SectionPrinter[]) {
    this.list = list;
    this.form.controls.section.setValue(this.sectionId);
    this.form.controls.saleCategories.clear();
    this.list.forEach((x) =>
      this.form.controls.saleCategories.push(
        new FormGroup({
          saleCategory: new FormControl<string>(x.saleCategory?.name ?? 'Default', {
            nonNullable: true,
          }),
          printer: new FormControl<string | null>(x.printer?.id ?? null),
          copies: new FormControl<number>(x.copies, { nonNullable: true }),
        }),
      ),
    );
  }

  save() {
    this.ser.save(this.sectionId, this.getItem()).subscribe(
      (result: SectionPrinter[]) => {
        this.toaster.show('Success', '');
        this.showItem(result);
      },
      (error) => {
        this.toaster.show('Error', error);
      },
    );
  }

  delete() {
    this.ser.delete(this.sectionId).subscribe(
      () => {
        this.toaster.show('Success', '');
      },
      (error) => {
        this.toaster.show('Error', error);
      },
    );
  }

  confirmDelete(): void {
    const dialogRef = this.dialog.open(ConfirmDialogComponent, {
      width: '250px',
      data: {
        title: 'Delete all printers for this section?',
        content: 'Are you sure? This cannot be undone.',
      },
    });

    dialogRef.afterClosed().subscribe((result: boolean) => {
      if (result) {
        this.delete();
      }
    });
  }

  getItem(): SectionPrinter[] {
    const formModel = this.form.value;
    this.sectionId = formModel.section ?? '';
    const array = this.form.controls.saleCategories;
    this.list.forEach((item, index) => {
      const cont = array.controls[index].value;
      if (cont.printer === null || cont.printer === undefined) {
        item.printer = null;
      } else {
        item.printer = { id: cont.printer };
      }
      item.copies = cont.copies ?? 0;
    });
    return this.list;
  }

  show(val: MatSelectChange) {
    this.router.navigate(['/section-printers', val.value]);
  }
}
